import os
import subprocess
import glob
import shutil
import urllib.request
from Bio.PDB import PDBParser
from tqdm import tqdm
import argparse
from main import vina, vina_split

def run_command(command):
    subprocess.run(command, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

def calculate_gridbox_center(structure):
    print(f"  Calculating GridBox Center")    
    model = structure[0]  # Get the first model from the PDB structure
    atoms = model.get_atoms()  # Get the list of atoms in the model
    
    x_sum = y_sum = z_sum = 0.0
    num_atoms = 0
    
    for atom in atoms:
        x, y, z = atom.get_coord()
        x_sum += x
        y_sum += y
        z_sum += z
        num_atoms += 1
    
    x_center = round(x_sum / num_atoms, 3)
    y_center = round(y_sum / num_atoms, 3)
    z_center = round(z_sum / num_atoms, 3)
    
    return x_center, y_center, z_center

def read_energy_and_save_to_csv(counter, output_model_dir, output_name, csv_path):    
    vina_output_file = os.path.join(output_model_dir, f"{output_name}.pdbqt")
        
    with open(vina_output_file, 'r') as vina_output:
        lines = vina_output.readlines()

    energy = None
    for line in lines:
        if line.startswith("REMARK VINA RESULT:"):
            energy = float(line.split()[3])
            break  # Stop reading after the first matching line

    if energy is not None:
        with open(csv_path, 'a') as csv_file:
            csv_file.write(f"{counter['value']}, {output_name.replace('output_', '').replace('_minimized', '').upper()}, {energy:.3f}\n")
    else:
        print(f"No REMARK VINA RESULT line found in the output of {os.path.basename(lig_pdbqt_path)}")

def process_docking_ligand_native(counter, lig_pdb_path, size_x, size_y, size_z, num_modes, exhaustiveness, cpu, num_iterations, csv_path, prepare_receptor_script, receptor_pdb_path, receptor_pdbqt_path, prepare_ligand_script, lig_pdbqt_path, directory, vina, output_model_dir):
    # Get ligand's center coordinates
    parser = PDBParser()
    structure = parser.get_structure("ligand", lig_pdb_path)
    x_center, y_center, z_center = calculate_gridbox_center(structure)

    # Create config.txt with updated values
    with open('config.txt', 'w') as config_file:
        config_file.write(f"size_x = {size_x}\n")
        config_file.write(f"size_y = {size_y}\n")
        config_file.write(f"size_z = {size_z}\n")
        config_file.write(f"center_x = {x_center}\n")
        config_file.write(f"center_y = {y_center}\n")
        config_file.write(f"center_z = {z_center}\n")
        config_file.write(f"num_modes = {num_modes}\n")
        config_file.write(f"exhaustiveness = {exhaustiveness}\n")
        config_file.write(f"cpu = {cpu}\n")
        config_file.write("# Script written by:\n")
        config_file.write("# La Ode Aman\n")
        config_file.write("# laodeaman.ai@gmail.com\n")
        config_file.write("# laode_aman@ung.ac.id\n")
        config_file.write("# Universitas Negeri Gorontalo, Indonesia\n")

    print("  Docking parameter:")
    print("\tsize_x =", size_x)
    print("\tsize_y =", size_y)
    print("\tsize_z =", size_z)
    print("\tcenter_x =", x_center)
    print("\tcenter_y =", y_center)
    print("\tcenter_z =", z_center)
    print("\tnum_modes =", num_modes)
    print("\texhaustiveness =", exhaustiveness)
    print("\tcpu =", cpu)

    # Create energy_summary.csv
    print("  Create energy summary as csv file")
    
    if not os.path.exists(csv_path):
        with open(csv_path, 'w') as csv_file:
            csv_file.write("No., Ligand ID (name), Binding Affinity (kcal per mol)\n")
    else:
        print("CSV file for summary energies is available")
      
    # Prepare receptor and native ligand
    run_command(f'{prepare_receptor_script} -r {receptor_pdb_path} -o {receptor_pdbqt_path}')
    run_command(f'{prepare_ligand_script} -l {lig_pdb_path} -o {lig_pdbqt_path}')
   
    counter ["value"] += 1
    
    # Run Vina
    print(f"  Docking and save energy")
    output_name = f"output_{directory}_lig"
    run_command(f'{vina} --receptor {receptor_pdbqt_path} --ligand {lig_pdbqt_path} --config config.txt --out {output_name}.pdbqt')

    # Move files
    os.rename(f"{output_name}.pdbqt", os.path.join(output_model_dir, f"{output_name}.pdbqt"))

    # Save to csv   
    read_energy_and_save_to_csv(counter, output_model_dir, output_name, csv_path)
            
def process_in_ligand_dir(ligand_dir, ligand_tmp_dir):
    os.chdir(ligand_dir)
    ligand_list = os.listdir(ligand_dir)
    ligand_files = [filename for filename in ligand_list if filename.endswith((".smi",".sdf", ".pdb", ".smiles", ".mol", ".mol2"))]
     
    for filename in ligand_files:
        ligand_test = os.path.splitext(os.path.basename(filename))[0]
        try:
            command = f"obabel {filename} -osmi -d -O {os.path.join(ligand_tmp_dir, ligand_test)}.smi -m"
            subprocess.run(command, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        except subprocess.CalledProcessError as e:
            print(f"An error occurred while preparing ligand: {filename}")

def process_in_main_ligand(main_ligand_dir, main_ligand_tmp):
    os.chdir(main_ligand_dir)
    ligand_list = os.listdir(main_ligand_dir)
    ligand_files = [filename for filename in ligand_list if filename.endswith((".smi",".sdf", ".pdb", ".smiles", ".mol", ".mol2"))]
     
    for filename in ligand_files:
        ligand_test = os.path.splitext(os.path.basename(filename))[0]
        try:
            command = f"obabel {filename} -osmi -d -O {os.path.join(main_ligand_tmp, ligand_test)}.smi -m"
            subprocess.run(command, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        except subprocess.CalledProcessError as e:
            print(f"An error occurred while preparing ligand: {filename}")

def process_in_main_ligand_tmp(main_ligand_tmp, num_iterations):
    os.chdir(main_ligand_tmp)    
    smi_files = [f for f in os.listdir(main_ligand_tmp) if f.endswith(".smi")]
    
    for smi_file in tqdm(smi_files, desc="Generating 3D conformation of main ligands"):
        with open(smi_file, 'r') as file:
            parts = file.readline().split()[1]
            file_name = parts.split('/')[-1]
            new_name = file_name.split('.')[0].upper()

        try:
            obabel_convert_command_mol = f'obabel {smi_file} -omol -h -O {new_name}_2d.mol --gen2d'
            subprocess.run(obabel_convert_command_mol, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
            
            obabel_convert_command_mol2 = f'obabel {new_name}_2d.mol -omol -h -O {new_name}.mol --gen3d'
            subprocess.run(obabel_convert_command_mol2, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        
           
        except subprocess.CalledProcessError as e:
            print(f"An error occurred while generating conformation of {new_name}")
        finally:
            os.remove(smi_file)
            os.remove(f"{new_name}_2d.mol")
     
    pdb_files = [f for f in os.listdir(main_ligand_tmp) if f.endswith(".mol")]
    
    for pdb_file in tqdm(pdb_files, desc="Geometry optimization of main ligands"):
        obminimize_input = pdb_file
        obminimize_output = f"{os.path.splitext(pdb_file)[0]}_minimized.pdb"
         
        try:
            obminimize_command = f'obminimize -ff Ghemical -o "pdb" {obminimize_input} > {obminimize_output}'
            subprocess.run(obminimize_command, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)             
            
        except subprocess.CalledProcessError as e:
            print(f"An error occurred while energy minimization of {obminimize_input}")
        finally:
            os.remove(obminimize_input)
  
def process_in_ligand_tmp_dir(ligand_tmp_dir, num_iterations):
    os.chdir(ligand_tmp_dir)    
    smi_files = [f for f in os.listdir(ligand_tmp_dir) if f.endswith(".smi")]
    
    for smi_file in tqdm(smi_files, desc="Generating 3D conformation of test ligands"):
        with open(smi_file, 'r') as file:
            parts = file.readline().split()[1]
            file_name = parts.split('/')[-1]
            new_name = file_name.split('.')[0].upper()

        try:
            obabel_convert_command_mol = f'obabel {smi_file} -omol -h -O {new_name}_2d.mol --gen2d'
            subprocess.run(obabel_convert_command_mol, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
            
            obabel_convert_command_mol2 = f'obabel {new_name}_2d.mol -omol -h -O {new_name}.mol --gen3d'
            subprocess.run(obabel_convert_command_mol2, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        
           
        except subprocess.CalledProcessError as e:
            print(f"An error occurred while generating conformation of {new_name}")
        finally:
            os.remove(smi_file)
            os.remove(f"{new_name}_2d.mol")
     
    pdb_files = [f for f in os.listdir(ligand_tmp_dir) if f.endswith(".mol")]
    
    for pdb_file in tqdm(pdb_files, desc="Geometry optimization of test ligands"):
        obminimize_input = pdb_file
        obminimize_output = f"{os.path.splitext(pdb_file)[0]}_minimized.pdb"
         
        try:
            obminimize_command = f'obminimize -ff Ghemical -o "pdb" {obminimize_input} > {obminimize_output}'
            subprocess.run(obminimize_command, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)             
            
        except subprocess.CalledProcessError as e:
            print(f"An error occurred while energy minimization of {obminimize_input}")
        finally:
            os.remove(obminimize_input)
     
def process_ligand_link(ligand_link_path, ligand_dir):
    for link_path in ligand_link_path:
        if os.path.exists(link_path):
            with open(link_path, 'r') as f:
                for line in f:
                    line = line.strip()
                    if line.startswith("http"):
                        ligand_url = line
                        try:
                            # Download the ligand file
                            ligand_file = os.path.join(ligand_dir, os.path.basename(ligand_url))
                            urllib.request.urlretrieve(ligand_url, ligand_file)
                            
                            # Membaca semua baris ke dalam list
                            with open(ligand_file, 'r') as file:
                                lines = file.readlines()

                            # Menulis kembali isi list ke file, mengabaikan baris pertama
                            with open(ligand_file, 'w') as file:
                                file.writelines(lines[1:])
                        except Exception as e:
                            print(f"An error occurred while downloading the url ligand: {ligand_url}")
                            print(f"Error details: {e}")
                        
def process_docking_ligand_test(main_ligand_tmp, ligand_tmp_dir, output_model_dir, counter, size_x, size_y, size_z, num_modes, exhaustiveness, cpu, num_iterations, csv_path, receptor_pdbqt_path, prepare_ligand_script, directory, vina):
    ligand_files = glob.glob(os.path.join(ligand_tmp_dir, '*.pdb'))
    num_ligand_files = len(ligand_files)
    
    main_ligand_files = glob.glob(os.path.join(main_ligand_tmp, '*.pdb'))
    num_main_files = len(main_ligand_files)
    print(f"  Total main ligand = {num_main_files}")
    print(f"  Total test ligand = {num_ligand_files}")
    print("  Using docking parameter from reference ligand")
    
    for main_ligand in main_ligand_files:
        main_name = os.path.splitext(os.path.basename(main_ligand))[0]
        main_dest_path = os.path.join(".", main_name + ".pdb")
        print(f"  Main ligand: {main_name}")
        for pdb_file in tqdm(ligand_files, desc="  Docking & save energy"):
            counter["value"] += 1
            filename = os.path.splitext(os.path.basename(pdb_file))[0]

            # Construct the destination path
            dest_path = os.path.join(".", filename + ".pdb")

            try:
                if not os.path.exists(dest_path):
                    # Copy pdb_file to the current directory
                    shutil.copy(main_ligand, main_dest_path)
                    shutil.copy(pdb_file, dest_path)

                # Convert *_lig.pdb to pdbqt
                main_pdbqt = f"{main_name}.pdbqt"
                run_command(f'{prepare_ligand_script} -l {main_ligand} -o {main_pdbqt}')
                
                pdbqt_file = f"{filename}.pdbqt"
                run_command(f'{prepare_ligand_script} -l {pdb_file} -o {pdbqt_file}')

                # Run Vina            
                output_name = f"output_{main_name}_{filename}"                            
                run_command(f'{vina} --receptor {receptor_pdbqt_path} --ligand {main_pdbqt} {pdbqt_file} --config config.txt --out {output_name}.pdbqt')
                
                # Move files
                os.rename(f"{output_name}.pdbqt", os.path.join(output_model_dir, f"{output_name}.pdbqt"))

                #Remove the original pdb_file and pdbqt_file
                # os.remove(main_dest_path)
                # os.remove(dest_path)
                # os.remove(main_pdbqt)
                # os.remove(pdbqt_file)

                # Save to CSV
                read_energy_and_save_to_csv(counter, output_model_dir, output_name, csv_path)            
                
            except Exception as e:
                print(f"An error occurred writing energy of {filename.replace('_minimized', '')}")

            except Exception as e:
                if os.path.exists(main_dest_path):
                    os.remove(main_dest_path)
                if os.path.exists(os.path.join(directory, f'{main_name}.pdbqt')):
                    os.remove(os.path.join(directory, f'{main_name}.pdbqt'))
                if os.path.exists(dest_path):
                    os.remove(dest_path)
                if os.path.exists(os.path.join(directory, f'{filename}.pdbqt')):
                    os.remove(os.path.join(directory, f'{filename}.pdbqt'))

def print_dev(developer_note, developer_contact, citation_list):
    print("")
    print(developer_note)
    print("")
    print(developer_contact)
    print("")
    print(citation_list)
    print("")

def main():  
    print_dev(developer_note, developer_contact, citation_list)
    current_directory = os.getcwd()
    os.chdir(current_directory)
          
    # Membaca isi file 'config_tmp.txt'
    with open('config_la2vina.txt', 'r') as config_file:
        config_lines = config_file.readlines()
    
    # Membuat kamus (dictionary) untuk menyimpan variabel-variabel
    config_variables = {}

    # Memproses setiap baris dalam file 'config.txt'
    for line in config_lines:
            # Mengabaikan baris yang dimulai dengan '#' (komentar)
            if not line.strip().startswith('#'):
                # Memisahkan nama variabel dan nilai variabel
                try:
                    name, value = line.strip().split('=')
                    # Membersihkan spasi dan karakter lainnya
                    name = name.strip()
                    value = value.strip()
                    # Menambahkan variabel ke dalam kamus
                    config_variables[name] = value
                except ValueError:
                    pass

    # Sekarang, Anda memiliki variabel-variabel dalam kamus config_variables yang dapat digunakan dalam skrip 'lavina.py'
    size_x = config_variables['size_x']
    size_y = config_variables['size_y']
    size_z = config_variables['size_z']
    num_modes = config_variables['num_modes']
    exhaustiveness = config_variables['exhaustiveness']
    cpu = config_variables['cpu']
    num_iterations = config_variables['num_iterations']    
    ligand_dir = os.path.join(current_directory, "ligand_input")
    ligand_tmp_dir = os.path.join(current_directory, "ligand_tmp")
    main_ligand_dir = os.path.join(current_directory, "main_ligand")
    main_ligand_tmp = os.path.join(current_directory, "main_ligand_tmp")    
    ligand_link_files = glob.glob(os.path.join(ligand_dir, "*ligand_link.txt"))
    
    mgl_directory = config_variables['mgl_directory']
    prepare_ligand = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "prepare_ligand4.py")
    prepare_receptor = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "prepare_receptor4.py")
    prepare_gpf = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "prepare_gpf4.py")
    prepare_dpf = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "prepare_dpf4.py")
    prepare_lowest_energy = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "write_lowest_energy_ligand.py")
    prepare_summarize_result = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "summarize_results4.py")
    
    if ligand_link_files:
        ligand_link_path = ligand_link_files
        
    else:
        ligand_link_path = []

    if os.path.exists(config_variables['vina']):
        vina = config_variables['vina']
    else:
        vina = "vina"
   
    prepare_ligand_script = os.path.join(mgltools_dir, "MGLToolsPckgs/AutoDockTools/Utilities24/prepare_ligand4.py")
    prepare_receptor_script = os.path.join(mgltools_dir, "MGLToolsPckgs/AutoDockTools/Utilities24/prepare_receptor4.py")
    ligand_tmp_dir = os.path.join(current_directory, 'ligand_tmp')
    main_ligand_tmp = os.path.join(current_directory, 'main_ligand_tmp')
    output_dir = os.path.join(current_directory, 'output')

    print("Removing unnecessary files and directories in the output target")

    # Create ligand_tmp directory if it doesn't exist
    if os.path.exists(ligand_tmp_dir):
        for item in os.listdir(ligand_tmp_dir):
            item_path = os.path.join(ligand_tmp_dir, item)
            if os.path.isfile(item_path):
                os.remove(item_path)
            elif os.path.isdir(item_path):
                shutil.rmtree(item_path)
    else:
        os.makedirs(ligand_tmp_dir, exist_ok=True)
        
    # Create main_ligand_tmp directory if it doesn't exist
    if os.path.exists(main_ligand_tmp):
        for item in os.listdir(main_ligand_tmp):
            item_path = os.path.join(main_ligand_tmp, item)
            if os.path.isfile(item_path):
                os.remove(item_path)
            elif os.path.isdir(item_path):
                shutil.rmtree(item_path)
    else:
        os.makedirs(main_ligand_tmp, exist_ok=True)

    # Create output directory if it doesn't exist
    if os.path.exists(output_dir):
        for item in os.listdir(output_dir):
            item_path = os.path.join(output_dir, item)
            if os.path.isfile(item_path):
                os.remove(item_path)
            elif os.path.isdir(item_path):
                shutil.rmtree(item_path)
    else:
        os.makedirs(output_dir, exist_ok=True)

    print("Getting ligands from online sources")
    process_ligand_link(ligand_link_path, ligand_dir)

    print("Preparing the test ligands")
    process_in_ligand_dir(ligand_dir, ligand_tmp_dir)
    process_in_ligand_tmp_dir(ligand_tmp_dir, num_iterations)
    process_in_main_ligand(main_ligand_dir, main_ligand_tmp)
    process_in_main_ligand_tmp(main_ligand_tmp, num_iterations)

    
    os.chdir(current_directory)
    
    for directory in glob.glob('model*'):
        if os.path.isdir(directory):
            counter = {"value": 0}
            print(f"\nProcessing {directory.upper()}")
            os.chdir(directory)
            
            receptor_pdb_path = f"{os.path.basename(directory)}_rec.pdb"
            receptor_pdbqt_path = f"{os.path.basename(directory)}_rec.pdbqt"
            lig_pdb_path = f"{os.path.basename(directory)}_lig.pdb"
            lig_pdbqt_path = f"{os.path.basename(directory)}_lig.pdbqt"
            lig_name = f"{os.path.basename(directory)}_lig"
            model_dir = f"output_{os.path.basename(directory)}"
            output_model_dir = os.path.join(output_dir, model_dir)
            os.makedirs(output_model_dir, exist_ok=True)
            csv_path = os.path.join(output_model_dir, 'energy_summary.csv')
                    
            
            if os.path.exists(receptor_pdb_path) or os.path.exists(lig_pdb_path) or os.path.exists(receptor_pdbqt_path) or os.path.exists(lig_pdbqt_path):
                print("Docking reference ligand:")
                print(f"  Receptor: {receptor_pdbqt_path}")
                print(f"  Ligand: {lig_pdbqt_path}")
                process_docking_ligand_native(counter, lig_pdb_path, size_x, size_y, size_z, num_modes, exhaustiveness, cpu, num_iterations, csv_path, prepare_receptor_script, receptor_pdb_path, receptor_pdbqt_path, prepare_ligand_script, lig_pdbqt_path, directory, vina, output_model_dir)
                print("Docking test ligands: ")
                print(f"  Receptor: {receptor_pdbqt_path}")        
                process_docking_ligand_test(main_ligand_tmp, ligand_tmp_dir, output_model_dir, counter, size_x, size_y, size_z, num_modes, exhaustiveness, cpu, num_iterations, csv_path, receptor_pdbqt_path, prepare_ligand_script, directory, vina)
            else:
                print(f"Skipping docking: receptor or reference ligand in {directory} not found.")
            
            os.chdir(current_directory)
            print(f"Success: {directory}")

    # Print developer's note, contact, and citation list
    print_dev(developer_note, developer_contact, citation_list)

if __name__ == "__main__":
  
    current_directory = (os.path.join(os.getcwd(), "LADOCK_la2vina"))
    
    if os.path.exists(current_directory):
        os.chdir(current_directory)
        main() 
    else:
        print("Your job directory (LADOCK_la2vina) is not ready. Please create it using:")
        print("ladock --create la2vina")



import os
import subprocess
import glob
import shutil
import urllib.request
from Bio.PDB import PDBParser
from tqdm import tqdm
import argparse
import gzip
import csv
from main import *
import concurrent.futures
from concurrent.futures import as_completed
from rdkit import Chem
from rdkit.Chem import AllChem

def run_command(command):
    subprocess.run(command, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

def calculate_gridbox_center(structure):
    print(f"  Calculating GridBox Center")    
    model = structure[0]  # Get the first model from the PDB structure
    atoms = model.get_atoms()  # Get the list of atoms in the model
    
    x_sum = y_sum = z_sum = 0.0
    num_atoms = 0
    
    for atom in atoms:
        x, y, z = atom.get_coord()
        x_sum += x
        y_sum += y
        z_sum += z
        num_atoms += 1
    
    x_center = round(x_sum / num_atoms, 3)
    y_center = round(y_sum / num_atoms, 3)
    z_center = round(z_sum / num_atoms, 3)
    
    return x_center, y_center, z_center

def read_energy_and_save_to_csv(counter, output_model_dir, lig_name, csv_path):    
    vina_output_file = os.path.join(output_model_dir, f"output_{lig_name}.pdbqt")
        
    with open(vina_output_file, 'r') as vina_output:
        lines = vina_output.readlines()

    energy = None
    for line in lines:
        if line.startswith("REMARK VINA RESULT:"):
            energy = float(line.split()[3])
            break  # Stop reading after the first matching line

    if energy is not None:
        with open(csv_path, 'a') as csv_file:
            csv_file.write(f"{counter['value']}, {lig_name.replace('output_', '').replace('_minimized', '').upper()}, {energy:.3f}\n")
    else:
        print(f"No REMARK VINA RESULT line found in the output of {os.path.basename(lig_pdbqt_path)}")

def process_docking_ligand_native(counter, size_x, size_y, size_z, num_modes, exhaustiveness, max_workers, num_iterations, csv_path, prepare_receptor, receptor_name, prepare_ligand, lig_name, directory, ensamble, output_model_dir, cpu):

    # Get ligand's center coordinates
    parser = PDBParser()
    lig_pdb_path = os.path.join(current_directory, directory, f"{lig_name}.pdb")

    structure = parser.get_structure("ligand", lig_pdb_path)
    x_center, y_center, z_center = calculate_gridbox_center(structure)

    # Create config.txt with updated values
    with open('config.txt', 'w') as config_file:
        config_file.write(f"size_x = {size_x}\n")
        config_file.write(f"size_y = {size_y}\n")
        config_file.write(f"size_z = {size_z}\n")
        config_file.write(f"center_x = {x_center}\n")
        config_file.write(f"center_y = {y_center}\n")
        config_file.write(f"center_z = {z_center}\n")
        config_file.write(f"num_modes = {num_modes}\n")
        config_file.write(f"exhaustiveness = {exhaustiveness}\n")
        config_file.write(f"cpu = {cpu}\n")
        config_file.write("# Script written by:\n")
        config_file.write("# La Ode Aman\n")
        config_file.write("# laodeaman.ai@gmail.com\n")
        config_file.write("# laode_aman@ung.ac.id\n")
        config_file.write("# Universitas Negeri Gorontalo, Indonesia\n")

    print("  Docking parameter:")
    print("\tsize_x =", size_x)
    print("\tsize_y =", size_y)
    print("\tsize_z =", size_z)
    print("\tcenter_x =", x_center)
    print("\tcenter_y =", y_center)
    print("\tcenter_z =", z_center)
    print("\tnum_modes =", num_modes)
    print("\texhaustiveness =", exhaustiveness)
    print("\tcpu =", cpu)

    # Create energy_summary.csv
     
    if not os.path.exists(csv_path):
        with open(csv_path, 'w') as csv_file:
            csv_file.write("No., Ligand ID (name), Binding Affinity (kcal per mol)\n")
    else:
        pass

      
    # Prepare receptor and native ligand
    run_command(f'{prepare_receptor} -r {receptor_name}.pdb')
    run_command(f'{prepare_ligand} -l {lig_name}.pdb')
    
    # Run Vina
    print(f"  Docking and save energy")
    run_command(f'{ensamble} --receptor {receptor_name}.pdbqt --ligand {lig_name}.pdbqt --config config.txt --out output_{lig_name}.pdbqt')


    # Move files
    os.rename(f"output_{lig_name}.pdbqt", os.path.join(output_model_dir, f"output_{lig_name}.pdbqt"))

    
    # Save to csv   
    read_energy_and_save_to_csv(counter, output_model_dir, lig_name, csv_path)
         
def process_in_ligand_dir(ligand_file, ligand_dir, ligand_tmp_dir):    
    os.chdir(ligand_dir)
    ligand_name = os.path.basename(ligand_file)
    print(f"\n  Ligand: {ligand_name}")
    print(f"  Splitting {ligand_name} to single molecule")

    ligand_test = os.path.splitext(os.path.basename(ligand_file))[0]
    try:
        if ligand_file.endswith((".smiles", ".smi")):
            preparing_smi_file(ligand_file)            
            command = f"obabel {ligand_file} -osmi -O {os.path.join(ligand_tmp_dir, ligand_test)}.smi -m -h"
        elif ligand_file.endswith((".sdf", ".pdb", ".mol", ".mol2")):
            command = f"obabel {ligand_file} -osdf -O {os.path.join(ligand_tmp_dir, ligand_test)}.sdf -m -h"
        else:
            return  # Skip unsupported file formats

        subprocess.run(command, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError as e:
        print(f"An error occurred while preparing ligand: {ligand_file}")

def process_and_optimize_smi(prepare_ligand, smi_file, ligand_tmp_dir):
    try:
        with open(smi_file, 'r') as file:
            parts = file.readline().split()[1]
            file_name = parts.split('/')[-1]
            new_name = file_name.split('.')[0].upper()

        # Convert .smi to .mol
        obabel_convert_command_mol = f'obabel {smi_file} -omol -h -O {new_name}.mol --gen2d'
        subprocess.run(obabel_convert_command_mol, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

        # Perform geometry optimization (output: .pdb)
        obminimize_input = f"{new_name}.mol"
        obminimize_output = f"{new_name}_minimized.pdb"
        obminimize_command = f'obminimize -o "pdb" {obminimize_input} > {obminimize_output}'
        subprocess.run(obminimize_command, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        convert_to_pdbqt(obminimize_output, prepare_ligand)
        os.remove(smi_file)
        os.remove(obminimize_input)
        os.remove(obminimize_output)
    except subprocess.CalledProcessError as e:
        pass
        
def process_and_optimize_sdf(prepare_ligand, sdf_file, ligand_tmp_dir):
    try:
        with open(sdf_file, 'r') as file:
            new_name = file.readline().strip()
        # Perform geometry optimization (output: .pdb)
        obminimize_input = sdf_file
        obminimize_output = f"{new_name}_minimized.pdb"
        obminimize_command = f'obminimize -ff Ghemical -o "pdb" {obminimize_input} > {obminimize_output}'
        subprocess.run(obminimize_command, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        convert_to_pdbqt(obminimize_output, prepare_ligand)
        os.remove(obminimize_input)
        os.remove(obminimize_output)
    except subprocess.CalledProcessError as e:
        pass
        
def process_in_ligand_tmp_dir(prepare_ligand, ligand_tmp_file, ligand_tmp_dir, max_workers):
    os.chdir(ligand_tmp_dir)
    smi_futures = []
    sdf_futures = []
    smi_files = [f for f in os.listdir(ligand_tmp_dir) if f.endswith(".smi")]
    sdf_files = [f for f in os.listdir(ligand_tmp_dir) if f.endswith(".sdf")]
        
    if smi_files:  # Periksa apakah ada file .smi yang tersedia
            with concurrent.futures.ThreadPoolExecutor(max_workers=1) as executor:
                for smi_file in smi_files:
                    smi_futures.append(executor.submit(process_and_optimize_smi, prepare_ligand, smi_file, ligand_tmp_dir))
                
                for future in tqdm(concurrent.futures.as_completed(smi_futures), total=len(smi_futures), desc="  Geometry optimization"):
                    try:
                        future.result()
                    except Exception as e:
                        print(f"An error occurred while processing : {e}")
    
    elif sdf_files:  # Periksa apakah ada file .sdf yang tersedia
            with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
                for sdf_file in sdf_files:
                    sdf_futures.append(executor.submit(process_and_optimize_sdf, prepare_ligand, sdf_file, ligand_tmp_dir))
                
                for future in tqdm(concurrent.futures.as_completed(sdf_futures), total=len(sdf_futures), desc="  Geometry optimization"):
                    try:
                        future.result()
                    except Exception as e:
                        print(f"An error occurred while processing {e}")

def process_ligand_link(ligand_url, ligand_dir, ligand_tmp_dir, lig_name, num_iterations, max_workers, size_x, size_y, size_z, num_modes, exhaustiveness, csv_path, prepare_receptor, receptor_name, prepare_ligand, directory, ensamble, output_model_dir, counter, cpu):
    retry_count = 0
    max_retries = 10
    ligand_file = None  # Inisialisasi variabel ligand_file
    
    while retry_count < max_retries:
        try:
            # Download the ligand file
            ligand_file = os.path.join(ligand_dir, os.path.basename(ligand_url))
            urllib.request.urlretrieve(ligand_url, ligand_file)
            
            break
            
        except Exception as e:
            print(e)
            print(f"Retrying to download: {ligand_url}")
            retry_count += 1

    if retry_count == max_retries:
        print(f"Failed to download ligand after {max_retries} retries.")
    
    if ligand_file and ligand_file.endswith(".gz"):
        # Extract the .gz file
        extracted_file = os.path.splitext(ligand_file)[0]  # Remove .gz extension
        with gzip.open(ligand_file, 'rb') as gz_file, open(extracted_file, 'wb') as out_file:
            out_file.writelines(gz_file)
        
        # Remove the downloaded .gz file
        os.remove(ligand_file)
        
        # Update the ligand_file to the extracted file
        ligand_file = extracted_file
    
    # Read all lines into a list
    with open(ligand_file, 'r') as file:
        lines = file.readlines()
    
    # Write back the content to the file, excluding the first line
    with open(ligand_file, 'w') as file:
        file.writelines(lines[0:])
    
    if ligand_file.endswith(".smi"):
        preparing_smi_file(ligand_file)
    process_in_ligand_dir(ligand_file, ligand_dir, ligand_tmp_dir)            
    
    # Remove the downloaded ligand file
    os.remove(ligand_file)
    
    valid_extensions = (".sdf", ".smi")
    ligand_tmp_files = [f for f in os.listdir(ligand_tmp_dir) if any(f.endswith(ext) for ext in valid_extensions)]
    num_ligand_files = len(ligand_tmp_files)

    print(f"  Total molecules = {num_ligand_files}")                    
    print(f"  Using {lig_name}.pdb docking parameter as reference")
    
    for ligand_tmp_file in ligand_tmp_files:
        process_in_ligand_tmp_dir(prepare_ligand, ligand_tmp_file, ligand_tmp_dir, max_workers)        
        process_docking_ligand_test(ligand_tmp_dir, counter, size_x, size_y, size_z, num_modes, exhaustiveness, num_iterations, csv_path, prepare_receptor, receptor_name, prepare_ligand, lig_name, directory, ensamble, output_model_dir, cpu)

def preparing_smi_file(smi_file_path):
    # Membaca file .smi ke dalam list
    with open(smi_file_path, 'r') as file:
        lines = file.readlines()

    # Membuat list untuk menyimpan baris-baris yang valid
    valid_lines = []

    for line in lines:
        # Memeriksa validitas SMILES dengan RDKit
        mol = Chem.MolFromSmiles(line.strip())
        if mol is not None:
            valid_lines.append(line)

    # Menyimpan kembali file .smi yang hanya berisi baris-baris yang valid
    with open(smi_file_path, 'w') as file:
        file.writelines(valid_lines)

def convert_to_pdbqt(pdb_file, prepare_ligand):
    filename = os.path.splitext(os.path.basename(pdb_file))[0]
    pdbqt_file = f"{filename}.pdbqt"
    run_command(f'{prepare_ligand} -l {pdb_file}')
    return pdbqt_file

def process_docking_ligand(pdbqt_file, counter, prepare_ligand, receptor_name, ensamble, output_model_dir, csv_path):
    try:
        # preparing, docking and save energy        
        filename = os.path.splitext(os.path.basename(pdbqt_file))[0]      
        output_file = os.path.join(output_model_dir, f'output_{filename}.pdbqt')
        run_command(f'{ensamble} --receptor {receptor_name}.pdbqt --ligand {pdbqt_file} --config config.txt --out {output_file}')
        read_energy_and_save_to_csv(counter, output_model_dir, filename, csv_path)    
        os.remove(pdbqt_file)
    except Exception as e:
        print(f"An error occurred while processing a ligand: {e}")

def process_docking_ligand_test(ligand_tmp_dir, counter, size_x, size_y, size_z, num_modes, exhaustiveness, num_iterations, csv_path, prepare_receptor, receptor_name, prepare_ligand, lig_name, directory, ensamble, output_model_dir, max_workers):
    os.chdir(directory)
    ligand_files = glob.glob(os.path.join(ligand_tmp_dir, '*.pdbqt'))
    num_ligand_files = len(ligand_files)
    
    counter ["value"] += 1
    if num_ligand_files == 0:        
        return

    futures = []

    with concurrent.futures.ProcessPoolExecutor(max_workers=max_workers) as executor:
        for pdbqt_file in ligand_files:
            futures.append(executor.submit(
                process_docking_ligand, pdbqt_file, counter, prepare_ligand, receptor_name, ensamble, output_model_dir, csv_path))

        # Gunakan as_completed untuk mengawasi pemrosesan .sdf
        for future in tqdm(concurrent.futures.as_completed(futures), total=len(futures), desc="  Docking and save energy"):
            try:
                future.result()  # Menunggu dan memeriksa hasil tugas (tidak ada yang diabaikan)
            except Exception as e:
                print(f"An error occurred while processing a .sdf file: {e}")

def sort_and_rewrite_csv(csv_path):
    try:
        data = []
        with open(csv_path, 'r') as csv_file:
            csv_reader = csv.reader(csv_file)
            header = next(csv_reader)  # Baca header (baris pertama)
            for row in csv_reader:
                data.append(row)            

        # Mengurutkan data berdasarkan kolom kedua (Binding Affinity)
        sorted_data = sorted(data, key=lambda x: float(x[2]))  # Menggunakan lambda untuk mengakses kolom kedua (indeks 2) sebagai kunci pengurutan

        # Menulis data yang sudah diurutkan kembali ke file CSV
        with open(csv_path, 'w', newline='') as csv_file:
            csv_writer = csv.writer(csv_file)
            csv_file.write("No., Ligand ID (name), Binding Affinity (kcal per mol)\n")  # Menulis header kembali
            for index, row in enumerate(sorted_data, start=1):
                csv_writer.writerow([index] + row[1:])  # Menulis nomor urutan, nama, dan binding affinity

        print(f"Output CSV file is in the {csv_path}")                
        return True
    except Exception as e:
        print(f"Error while sorting and rewriting CSV: {e}")
        return False

def print_dev(developer_note, developer_contact, citation_list):
    print("")
    print(developer_note)
    print("")
    print(developer_contact)
    print("")
    print(citation_list)
    print("")

def main(): 
    print_dev(developer_note, developer_contact, citation_list)
    
    os.chdir(current_directory)
    config_file = 'config_lavina.txt'
    
    with open(config_file, 'r') as config_file:
        config_lines = config_file.readlines()
    
    config_variables = {}
    retry_count = 0
    
    for line in config_lines:
        if not line.strip().startswith('#'):
            try:
                name, value = line.strip().split('=')
                name = name.strip()
                value = value.strip()
                config_variables[name] = value
            except ValueError:
                pass

    ensamble = vina  # Ganti "vina" dengan string yang sesuai
    num_modes = int(config_variables['num_modes'])  # Konversi ke integer
    exhaustiveness = int(config_variables['exhaustiveness'])  # Konversi ke integer
    num_iterations = 1000
    cpu = os.cpu_count()
    max_workers = os.cpu_count()
    
    size_x = float(config_variables['size_x'])  
    size_y = float(config_variables['size_y'])  
    size_z = float(config_variables['size_z'])  
    ligand_dir = os.path.join(current_directory, 'ligand_input')
    ligand_tmp_dir = os.path.join(current_directory, 'ligand_tmp')
    output_dir = os.path.join(current_directory, 'output')
    
    mgl_directory = config_variables['mgl_directory']
    prepare_ligand = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "prepare_ligand4.py")    
    prepare_receptor = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "prepare_receptor4.py")
    prepare_gpf = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "prepare_gpf4.py")
    prepare_dpf = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "prepare_dpf4.py")
    prepare_lowest_energy = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "write_lowest_energy_ligand.py")
    prepare_summarize_result = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "summarize_results4.py")
    
    print("Removing unnecessary files and directories in job directory...")

    # Create ligand_tmp and output directory if it doesn't exist
    if os.path.exists(ligand_tmp_dir):
        shutil.rmtree(ligand_tmp_dir)
    os.makedirs(ligand_tmp_dir, exist_ok=True)    
    if os.path.exists(output_dir):
        shutil.rmtree(output_dir)
    os.makedirs(output_dir, exist_ok=True) 
    
    for directory in glob.glob(os.path.join(current_directory, 'model*')):
        if os.path.isdir(directory):
            counter = {"value": 0}
            dir_name = os.path.basename(directory)
            receptor_name = ""
            lig_name = ""
            print(f"\nProcessing {dir_name.upper()}")
            os.chdir(directory)
            receptor_files = glob.glob('*rec.pdb')
            if receptor_files:    
                receptor_name = os.path.splitext(os.path.basename(receptor_files[0]))[0]
            ligand_files = glob.glob('*lig.pdb')
            if ligand_files:    
                lig_name = os.path.splitext(os.path.basename(ligand_files[0]))[0]                
            output_model_dir = os.path.join(output_dir, dir_name)         
            os.makedirs(output_model_dir, exist_ok=True)
            csv_path = os.path.join(output_model_dir, 'energy_summary.csv')
                
            if os.path.exists(f"{receptor_name}.pdb") and os.path.exists(f"{lig_name}.pdb"):
                with open(csv_path, 'w') as csv_file:
                    csv_file.write("No., Ligand ID (name), Binding Affinity (kcal per mol)\n")
                os.chdir(directory)
                print("Docking reference ligand:")
                print(f"  Receptor: {receptor_name}.pdbqt")
                print(f"  Ligand: {lig_name}.pdbqt")
                process_docking_ligand_native(counter, size_x, size_y, size_z, num_modes, exhaustiveness, max_workers, num_iterations, csv_path, prepare_receptor, receptor_name, prepare_ligand, lig_name, directory, ensamble, output_model_dir, cpu)
                print("\nDocking test ligands: ")
                print(f"  Receptor: {receptor_name}.pdbqt")                  
                os.chdir(ligand_dir)
                
                valid_extensions = (".txt")
                ligand_link_files = [f for f in os.listdir(ligand_dir) if any(f.endswith(ext) for ext in valid_extensions)]
                for ligand_link_path in ligand_link_files:
                    if os.path.exists(ligand_link_path):
                        with open(ligand_link_path, 'r') as f:
                            lines = f.readlines()                        
                        for line in lines:
                            if line.strip():
                                if not line.startswith("#"):
                                    ligand_url = line.strip()
                                    process_ligand_link(ligand_url, ligand_dir, ligand_tmp_dir, lig_name, num_iterations, max_workers, size_x, size_y, size_z, num_modes, exhaustiveness, csv_path, prepare_receptor, receptor_name, prepare_ligand, directory, ensamble, output_model_dir, counter, cpu)
                    
                valid_extensions = (".sdf", ".smi", ".pdb", ".smiles", ".mol", ".mol2")
                ligand_files = [f for f in os.listdir(ligand_dir) if any(f.endswith(ext) for ext in valid_extensions)]
                for ligand_file in ligand_files: 
                    process_in_ligand_dir(ligand_file, ligand_dir, ligand_tmp_dir)
                    
                    valid_extensions = (".sdf", ".smi")
                    ligand_tmp_files = [f for f in os.listdir(ligand_tmp_dir) if any(f.endswith(ext) for ext in valid_extensions)]
                    num_ligand_files = len(ligand_tmp_files)
                    print(f"  Total molecules = {num_ligand_files}")                    
                    print(f"  Using {lig_name}.pdb docking parameter as reference")
                    for ligand_tmp_file in ligand_tmp_files:
                        process_in_ligand_tmp_dir(prepare_ligand, ligand_tmp_file, ligand_tmp_dir, max_workers)                        
                        process_docking_ligand_test(ligand_tmp_dir, counter, size_x, size_y, size_z, num_modes, exhaustiveness, num_iterations, csv_path, prepare_receptor, receptor_name, prepare_ligand, lig_name, directory, ensamble, output_model_dir, cpu)
                            
                sort_and_rewrite_csv(csv_path)               
                print(f"Success: {dir_name.upper()}")

            else:
                print(f"Skipping docking: receptor or reference ligand in {dir_name} not found.")
                os.chdir(current_directory)                

    # Print developer's note, contact, and citation listdir
    print_dev(developer_note, developer_contact, citation_list)

if __name__ == "__main__":
    current_directory =  (os.path.join(os.getcwd(), "LADOCK_lavina")) 
    if os.path.exists(current_directory):  
        main() 
    else:
        print("Your job directory (LADOCK_lavina) is not ready. Please create it using:")
        print("ladock --create lavina")



#!/usr/bin/env python

import os
import subprocess
import glob
import shutil
from datetime import datetime
from main import *

def run_pdb2gmx(receptor_files):
    for receptor_file in receptor_files:
        rec_name = os.path.splitext(receptor_file)[0]        
        command = f'gmx pdb2gmx -f {receptor_file} -o {rec_name}_NEW.pdb -ignh'
        
        try:
            subprocess.run(command, shell=True, check=True)
            print(f"Perintah GROMACS berhasil dijalankan untuk {receptor_file}.")
        except subprocess.CalledProcessError as e:
            print(f"Perintah GROMACS gagal dengan kode keluar {e.returncode} untuk {receptor_file}.")
        except Exception as e:
            print(f"Terjadi kesalahan: {str(e)}")
   
def process_mol2_files_in_directories(mol2_files):
    for mol2_file in mol2_files:
        lig_name = os.path.splitext(os.path.basename(mol2_file))[0]
        subprocess.run(['obabel', mol2_file, '-opdb', '-h', '-O', f'{lig_name}.pdb'])
        subprocess.run(['acpype', '-i', f'{lig_name}.pdb', '-c', 'gas'])        

def extract_ligand_id_from_mol2(mol2_file):
    lig_id = None
    with open(mol2_file, 'r') as file:
        for line in file:
            if line.startswith('@<TRIPOS>ATOM'):
                # Mencari baris yang berisi ID ligand
                lig_id_line = next(file)
                parts = lig_id_line.split()
                if len(parts) > 1:
                    lig_id = parts[7][:3]
                break
    print(f"Ligand ID extracted: {lig_id}")
    return lig_id

def create_complex_pdb(complex_pdb, receptor_new_files, mol2_files):
    complex_data = ""
    
    try:
        for receptor_new_file in receptor_new_files:
            with open(receptor_new_file, 'r') as rec_new:
                rec_data = rec_new.read()
                complex_data += rec_data

        for mol2_file in mol2_files:
            lig_name = os.path.splitext(os.path.basename(mol2_file))[0]
            ligand_pdb = f"{lig_name}.acpype/{lig_name}_NEW.pdb"
            with open(ligand_pdb, 'r') as ligand_file:
                ligand_data = ligand_file.read()
                complex_data += ligand_data

        with open(complex_pdb, 'w') as complex_file:
            complex_file.write(complex_data)
        print(f"File {complex_pdb} berhasil dibuat.")
    except Exception as e:
        print(f"Terjadi kesalahan: {str(e)}")

def remove_lines_from_pdb_file(input_pdb_file, output_pdb_file):
    try:
        with open(input_pdb_file, 'r') as input_file:
            lines = input_file.readlines()

        # Filter out lines containing "MODEL", "ENDMDL", and "REMARK"
        filtered_lines = [line for line in lines if not any(keyword in line for keyword in ["MODEL", "ENDMDL", "TER", "REMARK"])]

        with open(output_pdb_file, 'w') as output_file:
            output_file.writelines(filtered_lines)
        
        print(f"File {output_pdb_file} berhasil dibuat tanpa baris yang mengandung 'MODEL', 'ENDMDL', dan 'REMARK'.")
    except Exception as e:
        print(f"Terjadi kesalahan: {str(e)}")

def create_complex_itp(mol2_files):
    atomtype_itp = ""    
    moleculetype_itp = ""
    complex_itp = ""
    
    for mol2_file in mol2_files:
        extract_ligand_id_from_mol2(mol2_file)            
        lig_name = os.path.splitext(mol2_file)[0]
        ligand_itp = f"{lig_name}.acpype/{lig_name}_GMX.itp"         
        with open(ligand_itp, 'r') as f:        
            flag_atomtypes = False            
            flag_moleculetype = False            
            
            for line in f:                
                if '[ atomtypes ]' in line:
                    flag_atomtypes = True
                if not line.strip():
                    flag_atomtypes = False                
                if flag_atomtypes:
                        if line not in complex_itp:
                            atomtype_itp += line

                if '[ moleculetype ]' in line:                    
                    flag_moleculetype = True                
                if flag_moleculetype:
                    moleculetype_itp += line               
            complex_itp = atomtype_itp + "\n"                                 
            complex_itp += moleculetype_itp + "\n"    
    
    # Ganti nama ligand dengan ID sesuai indeks
    for mol2_file in mol2_files:
        lig_name = os.path.splitext(mol2_file)[0]        
        lig_id = extract_ligand_id_from_mol2(mol2_file)
        print(f"{lig_id} : {lig_name}")
        complex_itp = complex_itp.replace(lig_name, lig_id)
    
    with open('ligand.itp', 'w') as complex_itp_file:
        complex_itp_file.write(complex_itp)
    print("File ligand.itp berhasil dibuat.")

def create_topol_top(mol2_files, receptor_new_files):
    # Baris yang ingin ditambahkan
    new_lines1 = "; Include ligand parameters\n#include \"ligand.itp\"\n\n"
    new_lines2 = ""  # Inisialisasi string new_lines2
    new_lines3 = ""  # Inisialisasi string new_lines3

    for file in mol2_files:
        lig_id = extract_ligand_id_from_mol2(file)
        new_lines2 += f'#include "{lig_id}-posre.itp"\n'
        new_lines3 += f'{lig_id}\t\t\t1\n'

    with open("topol.top", 'r') as topol_file:
        lines = topol_file.readlines()

    index1 = None
    index2 = None

    for i, line in enumerate(lines):
        if line.strip() == "; Include forcefield parameters":
            index1 = i
        if line.strip() == "; Include Position restraint file":
            index2 = i

    if index1 is not None:
        # Menambahkan new_lines1 setelah baris yang sesuai
        lines.insert(index1 + 3, new_lines1)

    if index2 is not None:
        # Menambahkan new_lines2 setelah baris yang sesuai
        lines.insert(index2 + 4, new_lines2)

    # Menambahkan new_lines3 pada bagian terakhir
    lines.extend([new_lines3])

    # Menulis ulang isi file topol.top dengan baris tambahan
    with open("topol.top", 'w') as topol_file:
        topol_file.writelines(lines)
   
def generate_box_pdb(box_pdb, box_type, distance):
    try:
        subprocess.run(["gmx", "editconf", "-f", "complex_clean.pdb", "-o", box_pdb, "-bt", box_type, "-d", str(distance), "-c"])
        print(f"File {box_pdb} berhasil dibuat.")
    except subprocess.CalledProcessError:
        print("Gagal menjalankan perintah gmx editconf.") 

def solvate_system(box_pdb, spc216_gro, topol_top, solv_gro):
    try:
        subprocess.run(["gmx", "solvate", "-cp", box_pdb, "-cs", spc216_gro, "-p", topol_top, "-o", solv_gro])
        print(f"File {solv_gro} berhasil dibuat.")
    except subprocess.CalledProcessError:
        print("Gagal menjalankan perintah gmx solvate.")

def ionization(solv_gro, topol_top, ions_gro):
    try:
        # Generate TPR file
        subprocess.run(["gmx", "grompp", "-f", "ions.mdp", "-c", solv_gro, "-p", topol_top, "-o", "ions.tpr", "-maxwarn", "1"])

        # Run genion for ionization
        genion_cmd = f"gmx genion -s ions.tpr -o {ions_gro} -p {topol_top} -pname NA -nname CL -neutral"
        subprocess.run(["bash", "-c", genion_cmd])
        
        print(f"File {ions_gro} berhasil dibuat.")
    except subprocess.CalledProcessError:
        print("Gagal menjalankan perintah gmx genion.")
 
def minimization(ions_gro, topol_top):
    try:
        # Generate TPR file for minimization
        subprocess.run(["gmx", "grompp", "-f", "em.mdp", "-c", ions_gro, "-p", topol_top, "-o", "em.tpr", "-maxwarn", "1"])
        
        # Run energy minimization using mdrun
        subprocess.run(["gmx", "mdrun", "-v", "-deffnm", "em"])
        
        # Calculate potential energy
        subprocess.run(["echo", "-e", "10\n0", "|", "gmx", "energy", "-f", "em.edr", "-o", "potential.xvg"])
        
        print("Minimization completed.")
    except subprocess.CalledProcessError:
        print("Gagal menjalankan perintah GROMACS.")
 
def make_index_system(em_gro, index_ndx):
    try:
        # Create index file for the system
        subprocess.run(["gmx", "make_ndx", "-f", em_gro, "-o", index_ndx])
        
        print("Index file created.")
    except subprocess.CalledProcessError:
        print("Gagal menjalankan perintah GROMACS.")

def copy_mdp_files(current_directory, subdir):
    mdp_files = glob.glob(os.path.join(current_directory, "*.mdp"))    
    try:
        for mdp_file in mdp_files:
            mdp_filename = os.path.basename(mdp_file)
            destination_path = os.path.join(".", mdp_filename)
            shutil.copyfile(mdp_file, mdp_filename)
        
        print("MDP files copied successfully.")
    except Exception as e:
        print(f"An error occurred: {str(e)}")
        # Handle the exception as needed
    finally:
        # Optionally, perform cleanup or additional actions here
        pass

def process_ligand_restraint(mol2_files):
    for mol2_file in mol2_files:
        lig_name = os.path.splitext(os.path.basename(mol2_file))[0]
        lig_id = extract_ligand_id_from_mol2(mol2_file)
        
        # Mencari semua file ligand_new_pdb dalam direktori yang sesuai
        ligand_new_pdbs = glob.glob(f"{lig_name}.acpype/{lig_name}_NEW.pdb")

        for ligand_new_pdb in ligand_new_pdbs:
            try:
                # Membuat indeks untuk ligand
                make_ndx_command = [
                    'gmx', 'make_ndx',
                    '-f', ligand_new_pdb,
                    '-o', f'{lig_id}-index.ndx'
                ]
                subprocess.run(make_ndx_command, check=True)

                # Menjalankan perintah gmx genrestr
                genrestr_command = [
                    'gmx', 'genrestr',
                    '-f', ligand_new_pdb,
                    '-n', f'{lig_id}-index.ndx',
                    '-o', f'{lig_id}-posre.itp',
                    '-fc', '1000', '1000', '1000'
                ]
                subprocess.run(genrestr_command, check=True)

                print(f"File {lig_id}-posre.itp berhasil dibuat untuk {ligand_new_pdb}")
            except subprocess.CalledProcessError as e:
                print(f"Terjadi kesalahan saat menjalankan perintah: {str(e)}")
            
def replace_text_in_files(mol2_files):
    mdp_files = glob.glob(os.path.join(".", "*.mdp"))    
    combined_lig_id = "_".join(extract_ligand_id_from_mol2(file) for file in mol2_files)    
    for mdp_file in mdp_files:
        with open(mdp_file, 'r') as file:
            file_data = file.read()            
            file_data = file_data.replace("Protein_UNK", f"Protein_{combined_lig_id}")

        with open(mdp_file, 'w') as file:
            file.write(file_data)

def run_nvt_simulation():
    # Jalankan grompp untuk NVT
    grompp_cmd = [
        'gmx', 'grompp',
        '-f', 'nvt.mdp',
        '-c', 'em.gro',
        '-r', 'em.gro',
        '-p', 'topol.top',
        '-n', 'index.ndx',
        '-o', 'nvt.tpr',
        '-maxwarn', '5'
    ]
    subprocess.run(grompp_cmd, check=True)

    # Jalankan mdrun untuk NVT
    mdrun_cmd = [
        'gmx', 'mdrun',
        '-v',
        '-s', 'nvt.tpr',
        '-deffnm', 'nvt'
    ]
    subprocess.run(mdrun_cmd, check=True)

def run_npt_simulation():
    # Jalankan grompp untuk NPT
    grompp_cmd = [
        'gmx', 'grompp',
        '-f', 'npt.mdp',
        '-c', 'nvt.gro',
        '-t', 'nvt.cpt',
        '-r', 'nvt.gro',
        '-p', 'topol.top',
        '-n', 'index.ndx',
        '-o', 'npt.tpr',
        '-maxwarn', '5'
    ]
    subprocess.run(grompp_cmd, check=True)

    # Jalankan mdrun untuk NPT
    mdrun_cmd = [
        'gmx', 'mdrun',
        '-v',
        '-s', 'npt.tpr',
        '-deffnm', 'npt'
    ]
    subprocess.run(mdrun_cmd, check=True)

def run_production_simulation():
    # Jalankan grompp untuk produksi
    grompp_cmd = [
        'gmx', 'grompp',
        '-f', 'md.mdp',
        '-c', 'npt.gro',
        '-t', 'npt.cpt',
        '-p', 'topol.top',
        '-n', 'index.ndx',
        '-o', 'md.tpr',
        '-maxwarn', '5'
    ]
    subprocess.run(grompp_cmd, check=True)

    # Jalankan mdrun untuk produksi
    mdrun_cmd = [
        'gmx', 'mdrun',
        '-v',
        '-s', 'md.tpr',
        '-deffnm', 'md'
    ]
    subprocess.run(mdrun_cmd, check=True)

def print_dev(developer_note, developer_contact, citation_list):
    print("")
    print(developer_note)
    print("")
    print(developer_contact)
    print("")
    print(citation_list)
    print("")

def main():
    print_dev(developer_note, developer_contact, citation_list)
    # Membaca isi file 'config_tmp.txt'
    with open('config_gmxprolig.txt', 'r') as config_file:
        config_lines = config_file.readlines()
    
    # Membuat kamus (dictionary) untuk menyimpan variabel-variabel
    config_variables = {}

    # Memproses setiap baris dalam file 'config.txt'
    for line in config_lines:
            # Mengabaikan baris yang dimulai dengan '#' (komentar)
            if not line.strip().startswith('#'):
                # Memisahkan nama variabel dan nilai variabel
                try:
                    name, value = line.strip().split('=')
                    # Membersihkan spasi dan karakter lainnya
                    name = name.strip()
                    value = value.strip()
                    # Menambahkan variabel ke dalam kamus
                    config_variables[name] = value
                except ValueError:
                    pass

    box_type = config_variables['box_type']
    distance = config_variables['distance']
    spc216_gro = config_variables['coordinate_file']
    current_directory = os.getcwd()  
    complex_pdb = "complex.pdb"  
    box_pdb = "box.pdb"    
    topol_top = "topol.top"    
    solv_gro = "solv.gro" 
    ions_gro = "ions.gro"
    em_gro = "em.gro"
    index_ndx = "index.ndx"
    
    for subdir in os.listdir(current_directory):
        if os.path.isdir(subdir):
            os.chdir(subdir)         
            mol2_files = glob.glob('*.mol2')
            receptor_files = glob.glob('*rec.pdb')            
            copy_mdp_files(current_directory, subdir)       
            replace_text_in_files(mol2_files)
            process_mol2_files_in_directories(mol2_files) 
            run_pdb2gmx(receptor_files)
            receptor_new_files = glob.glob('*_NEW.pdb')                     
            create_complex_pdb(complex_pdb, receptor_new_files, mol2_files)  
            remove_lines_from_pdb_file(complex_pdb, "complex_clean.pdb")            
            create_complex_itp(mol2_files)
            create_topol_top(mol2_files, receptor_new_files)
            generate_box_pdb(box_pdb, box_type, distance)
            solvate_system(box_pdb, spc216_gro, topol_top, solv_gro)
            ionization(solv_gro, topol_top, ions_gro)
            minimization(ions_gro, topol_top)
            process_ligand_restraint(mol2_files)
            make_index_system(em_gro, index_ndx)            
            os.chdir('..')    
            
    for subdir in os.listdir(current_directory):
        if os.path.isdir(subdir):
            os.chdir(subdir)        
            run_nvt_simulation()
            run_npt_simulation()
            run_production_simulation()            
            os.chdir('..')
            
    print_dev(developer_note, developer_contact, citation_list)        
if __name__ == "__main__":
    current_directory =  (os.path.join(os.getcwd(), "LADOCK_gmxprolig"))      
    
    if os.path.exists(current_directory):
        os.chdir(current_directory)
        main() 
    else:
        print("Your job directory (LADOCK_gmxprolig) is not ready. Please create it using:")
        print("ladock --create gmxprolig")



config_content = """# Silakan melakukan editing sesuai kebutuhan dan kenyataan Anda
size_x = 40
size_y = 40
size_z = 40
num_modes = 8
exhaustiveness = 8
cpu = 12
num_iterations = 1000
mgl_directory = /home/<user>/MGLTools-1.5.x

# This directory structure has been created using LADOCK. Follow these steps to prepare your input files and execute the simulations:
# 1. Place your test ligand files in the 'ligand_input' directory.
# 2. Place your main ligand files in the 'main_ligand' directory.
# 3. Put your receptor (model_*_rec.pdb) and reference ligand (model_*_lig.pdb) files in 'model_*' directories.
# 4. Ensure that your ligand files (test and main) are in one of the following formats: smi, smiles, mol, mol2, pdb, or sdf.
# 5. For ligands obtained from online sources, add the link to 'ligand_link.txt' and place it in the 'ligand_input' directory.
# 6. Adjust the parameters in the 'config_la2vina.txt' file in the job directory as needed. Make sure to correctly set the MGLTools installation directory and docking parameters in this file.
# 7. Execute the following command in the LADOCK_la2vina directory: `ladock --run la2vina`. 
# """

ligand_link_default = """# Add download links here. One link per line. Supported download link formats include: smi, smiles, mol, mol2, pdb, or sdf.
# Here's an example link (remove this link and add your own ligand file links):
# http://files.docking.org/2D/BA/BAAA.smi
# http://files.docking.org/2D/BA/BAAB.smi
"""



config_content = """# Silakan melakukan editing sesuai kebutuhan dan kenyataan Anda
# Nilai untuk -bt (box type):
box_type = triclinic

# Nilai untuk -d (distance)
distance = 1.2  

# Nilai untuk -cs (coordinate file for solvent):
coordinate_file = spc216.gro

# This directory structure has been created using LADOCK. Follow these steps to prepare your input files and execute the simulations:

# 1. Place each ligands (separated files) in the "complex" folder.
# 2. Make necessary edits to the mdp files in the job directory as needed .
# 3. Ensure that each ligand file in .pdb format.
# 4. Adjust the parameters in the 'config_gmxliglig.txt' file in the job directory as needed.
# 5. Verify that the required dependencies, such as acpype, openbabel, and gromacs, are installed and functioning properly.
# 6. Execute the following command: "ladock --run gmxlilig"
"""



config_content = """# Silakan melakukan editing sesuai kebutuhan dan kenyataan Anda
# Nilai untuk -bt (box type):
box_type = triclinic

# Nilai untuk -d (distance)
distance = 1.2

# Nilai untuk -cs (coordinate file for solvent):
coordinate_file = spc216.gro

# This directory structure has been created using LADOCK. Follow these steps to prepare your input files and execute the simulations:

# 1. Place each receptor and ligand(s) (separate files) in the each "complex" directory.
# 2. Make necessary edits to the mdp files in the job directory  as needed.
# 3. Ensure that receptor file in .pdb format.
# 4. Ensure that ligand file(s) in .mol format.
# 5. Adjust the parameters in the 'config_gmxprolig.txt' file in the job directory as needed.
# 6. Verify that the required dependencies, such as acpype, openbabel, and gromacs, are installed and functioning properly.
# 7. Execute the following command: "ladock --run gmxprolig"
"""




config_content = """# Silakan melakukan editing sesuai kebutuhan dan kenyataan Anda
size_x = 40
size_y = 40
size_z = 40
num_modes = 8
exhaustiveness = 8
mgl_directory = /home/<user>/MGLTools-1.5.x

# This directory structure has been created using LADOCK.
# Follow these steps to prepare your input files and execute the simulations:
# 1. Put your ligand files in the 'ligand_input' directory.
# 2. Put your receptor (*rec.pdb) dan reference ligand (*lig.pdb) files in 'model_*' directories.
# 3. Ensure your ligand input files are in formats: smi, smiles, mol, mol2, pdb, or sdf.
# 4. For ligands from online sources, add the link to 'ligand_link.txt' and put it in the 'ligand_input' directory.
# 5. Adjust the parameters in the 'gmxprolig_config.txt' file in the job directory as needed.
# 6. Adjust the parameters in the 'config_lavina.txt' file as needed. Set the MGLTools installation directory and docking parameters in this file correctly.
# 6. Execute the following command in the LADOCK_lavina directory: `ladock --run lavina`. 
"""

ligand_link_default = """# Add download links here. One link per line. Supported download link formats include: smi, smiles, mol, mol2, pdb, or sdf.
# Here's an example link (remove this link and add your own ligand file links):
# http://files.docking.org/2D/BA/BAAA.smi
# http://files.docking.org/2D/BA/BAAB.smi
"""



#!/usr/bin/env python
from setuptools import setup

setup(
    name="ladock",
    version="0.0.8",
    description="LADOCK is an innovative and free tool designed for conducting simultaneous simulations in computer-aided drug discovery, encompassing molecular docking and molecular dynamics. In molecular docking, LADOCK excels in handling single or double ligands. It supports ligands from various online sources. In molecular dynamics, LADOCK efficiently manages protein-ligand interactions and even ligand-ligand interactions, accommodating scenarios with one or multiple proteins and ligands.",
    author="La Ode Aman",
    author_email="laode_aman@ung.ac.id",
    license="Apache License 2.0",
    packages=["ladock"],
    python_requires='>=3.6',    
    include_package_data=True,
    entry_points={
        'console_scripts': ['ladock=ladock.main:main'],
    },
    install_requires=[
        'requests',
        'biopython',
        'tqdm',
        'argparse',
        'numpy',
        'scipy',
        'futures',
    ],
)




import os
import subprocess
import glob
import shutil
import urllib.request
from Bio.PDB import PDBParser
from tqdm import tqdm
import argparse
import gzip
import csv
from main import *
import concurrent.futures
from concurrent.futures import as_completed
from rdkit import Chem
from rdkit.Chem import AllChem
from pathlib import Path

def run_command(command):
    subprocess.run(command, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

def read_energy_and_save_to_csv(counter, output_file, csv_path, lig_name):
    # Buka file untuk membaca
    with open(output_file, 'r') as file:
        content = file.read()

    # Cari baris yang mengandung "RMSD TABLE"
    in_rmsd_table = False
    for line in content.split('\n'):
        if "RMSD TABLE" in line:
            in_rmsd_table = True
            continue
        if in_rmsd_table:
            if line.strip():
                words = line.split()
                try:
                    energy = float(words[3])
                    break
                except (ValueError, IndexError):
                    pass
    else:
        print("Tabel RMSD tidak ditemukan atau tidak ada nilai energy pada baris pertama kolom keempat.")

    # Jika energy ditemukan, simpan ke dalam CSV
    if energy is not None:
        with open(csv_path, 'a') as csv_file:
            csv_file.write(f"{counter['value']}, {lig_name.replace('_minimized', '').upper()}, {energy:.3f}\n")

def process_docking_ligand_native(prepare_gpf, prepare_dpf, counter, spacing, npts, max_workers, num_iterations, csv_path, prepare_receptor, receptor_name, prepare_ligand, lig_name, directory, ensamble, output_model_dir, parameter):

    # Create energy_summary.csv
    print("  Docking and save energy ...")
    if not os.path.exists(csv_path):
        with open(csv_path, 'w') as csv_file:
            csv_file.write("No., Ligand ID (name), Binding Affinity (kcal per mol)\n")
    else:
        pass

    counter ["value"] += 1
    
    # Prepare receptor & native, gpf, autogrid4, dpf, autodock4
    output_file = os.path.join(output_model_dir, f"output_{lig_name}.dlg")
    
    run_command(f'{prepare_ligand} -l {lig_name}.pdb')    
    run_command(f'{prepare_receptor} -r {receptor_name}.pdb -o {lig_name}_{receptor_name}.pdbqt -A checkhydrogens') 
    run_command(f'{prepare_gpf} -l {lig_name}.pdbqt -r {lig_name}_{receptor_name}.pdbqt -y -o {lig_name}_{receptor_name}.gpf')
    run_command(f'autogrid4 -p {lig_name}_{receptor_name}.gpf -l {lig_name}_{receptor_name}.glg')
    run_command(f'{prepare_dpf} -l {lig_name}.pdbqt -r {lig_name}_{receptor_name}.pdbqt {parameter} -o {lig_name}_{receptor_name}.dpf')
    run_command(f'{ensamble} -p {lig_name}_{receptor_name}.dpf -l {output_file}')  

    # Save to csv   
    read_energy_and_save_to_csv(counter, output_file, csv_path, lig_name)
         
def process_in_ligand_dir(ligand_file, ligand_dir, ligand_tmp_dir):    
    os.chdir(ligand_dir)
    ligand_name = os.path.basename(ligand_file)
    print(f"\n  Ligand: {ligand_name}")
    print(f"  Splitting {ligand_name} to single molecule")

    ligand_test = os.path.splitext(os.path.basename(ligand_file))[0]
    try:
        if ligand_file.endswith((".smiles", ".smi")):
            preparing_smi_file(ligand_file)            
            command = f"obabel {ligand_file} -osmi -O {os.path.join(ligand_tmp_dir, ligand_test)}.smi -m -h"
        elif ligand_file.endswith((".sdf", ".pdb", ".mol", ".mol2")):
            command = f"obabel {ligand_file} -osdf -O {os.path.join(ligand_tmp_dir, ligand_test)}.sdf -m -h"
        else:
            return  # Skip unsupported file formats

        subprocess.run(command, shell=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
    except subprocess.CalledProcessError as e:
        print(f"An error occurred while preparing ligand: {ligand_file}")

def process_and_optimize_smi(prepare_ligand, smi_file, ligand_tmp_dir):
    try:
        with open(smi_file, 'r') as file:
            parts = file.readline().split()[1]
            file_name = parts.split('/')[-1]
            new_name = file_name.split('.')[0].upper()

        # Convert .smi to .mol
        obabel_convert_command_mol = f'obabel {smi_file} -omol -h -O {new_name}.mol --gen2d'
        subprocess.run(obabel_convert_command_mol, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)

        # Perform geometry optimization (output: .pdb)
        obminimize_input = f"{new_name}.mol"
        obminimize_output = f"{new_name}_minimized.pdb"
        obminimize_command = f'obminimize -o "pdb" {obminimize_input} > {obminimize_output}'
        subprocess.run(obminimize_command, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        convert_to_pdbqt(obminimize_output, prepare_ligand)
        os.remove(smi_file)
        os.remove(obminimize_input)
        os.remove(obminimize_output)
    except subprocess.CalledProcessError as e:
        pass
        
def process_and_optimize_sdf(prepare_ligand, sdf_file, ligand_tmp_dir):
    try:
        with open(sdf_file, 'r') as file:
            new_name = file.readline().strip()
        # Perform geometry optimization (output: .pdb)
        obminimize_input = sdf_file
        obminimize_output = f"{new_name}_minimized.pdb"
        obminimize_command = f'obminimize -ff Ghemical -o "pdb" {obminimize_input} > {obminimize_output}'
        subprocess.run(obminimize_command, shell=True, check=True, stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
        convert_to_pdbqt(obminimize_output, prepare_ligand)
        os.remove(obminimize_input)
        os.remove(obminimize_output)
    except subprocess.CalledProcessError as e:
        pass
        
def process_in_ligand_tmp_dir(prepare_ligand, ligand_tmp_file, ligand_tmp_dir, max_workers):
    os.chdir(ligand_tmp_dir)
    smi_futures = []
    sdf_futures = []
    smi_files = [f for f in os.listdir(ligand_tmp_dir) if f.endswith(".smi")]
    sdf_files = [f for f in os.listdir(ligand_tmp_dir) if f.endswith(".sdf")]
        
    if smi_files:  # Periksa apakah ada file .smi yang tersedia
            with concurrent.futures.ThreadPoolExecutor(max_workers=1) as executor:
                for smi_file in smi_files:
                    smi_futures.append(executor.submit(process_and_optimize_smi, prepare_ligand, smi_file, ligand_tmp_dir))
                
                for future in tqdm(concurrent.futures.as_completed(smi_futures), total=len(smi_futures), desc="  Geometry optimization"):
                    try:
                        future.result()
                    except Exception as e:
                        print(f"An error occurred while processing : {e}")
    
    elif sdf_files:  # Periksa apakah ada file .sdf yang tersedia
            with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
                for sdf_file in sdf_files:
                    sdf_futures.append(executor.submit(process_and_optimize_sdf, prepare_ligand, sdf_file, ligand_tmp_dir))
                
                for future in tqdm(concurrent.futures.as_completed(sdf_futures), total=len(sdf_futures), desc="  Geometry optimization"):
                    try:
                        future.result()
                    except Exception as e:
                        print(f"An error occurred while processing {e}")

def process_ligand_link(prepare_gpf, prepare_dpf, ligand_url, ligand_dir, ligand_tmp_dir, lig_name, num_iterations, max_workers, spacing, npts, csv_path, prepare_receptor, receptor_name, prepare_ligand, directory, ensamble, output_model_dir, counter, parameter):
    retry_count = 0
    max_retries = 10
    ligand_file = None  # Inisialisasi variabel ligand_file
    
    while retry_count < max_retries:
        try:
            # Download the ligand file
            ligand_file = os.path.join(ligand_dir, os.path.basename(ligand_url))
            urllib.request.urlretrieve(ligand_url, ligand_file)
            
            break
            
        except Exception as e:
            print(e)
            print(f"Retrying to download: {ligand_url}")
            retry_count += 1

    if retry_count == max_retries:
        print(f"Failed to download ligand after {max_retries} retries.")
    
    if ligand_file and ligand_file.endswith(".gz"):
        # Extract the .gz file
        extracted_file = os.path.splitext(ligand_file)[0]  # Remove .gz extension
        with gzip.open(ligand_file, 'rb') as gz_file, open(extracted_file, 'wb') as out_file:
            out_file.writelines(gz_file)
        
        # Remove the downloaded .gz file
        os.remove(ligand_file)
        
        # Update the ligand_file to the extracted file
        ligand_file = extracted_file
    
    # Read all lines into a list
    with open(ligand_file, 'r') as file:
        lines = file.readlines()
    
    # Write back the content to the file, excluding the first line
    with open(ligand_file, 'w') as file:
        file.writelines(lines[0:])
    
    if ligand_file.endswith(".smi"):
        preparing_smi_file(ligand_file)
    process_in_ligand_dir(ligand_file, ligand_dir, ligand_tmp_dir)            
    
    # Remove the downloaded ligand file
    os.remove(ligand_file)
    
    valid_extensions = (".sdf", ".smi")
    ligand_tmp_files = [f for f in os.listdir(ligand_tmp_dir) if any(f.endswith(ext) for ext in valid_extensions)]
    num_ligand_files = len(ligand_tmp_files)

    print(f"  Total molecules = {num_ligand_files}")                    
    print(f"  Using {lig_name}.pdb docking parameter as reference")
    
    for ligand_tmp_file in ligand_tmp_files:
        process_in_ligand_tmp_dir(prepare_ligand, ligand_tmp_file, ligand_tmp_dir, max_workers)        
        process_docking_ligand_test(counter, prepare_receptor, prepare_ligand, prepare_gpf, prepare_dpf,  lig_name, output_model_dir, receptor_name, ligand_tmp_dir, csv_path, parameter, ensamble, directory, max_workers)

def preparing_smi_file(smi_file_path):
    # Membaca file .smi ke dalam list
    with open(smi_file_path, 'r') as file:
        lines = file.readlines()

    # Membuat list untuk menyimpan baris-baris yang valid
    valid_lines = []

    for line in lines:
        # Memeriksa validitas SMILES dengan RDKit
        mol = Chem.MolFromSmiles(line.strip())
        if mol is not None:
            valid_lines.append(line)

    # Menyimpan kembali file .smi yang hanya berisi baris-baris yang valid
    with open(smi_file_path, 'w') as file:
        file.writelines(valid_lines)

def convert_to_pdbqt(pdb_file, prepare_ligand):
    filename = os.path.splitext(os.path.basename(pdb_file))[0]
    pdbqt_file = f"{filename}.pdbqt"
    run_command(f'{prepare_ligand} -l {pdb_file}')
    return pdbqt_file

def process_docking_ligand(directory, prepare_gpf, prepare_dpf, pdbqt_file, counter, prepare_receptor, receptor_name, lig_name, parameter, ensamble, output_model_dir, csv_path):
    try:
        filename = os.path.splitext(os.path.basename(pdbqt_file))[0]
        output_file = os.path.join(output_model_dir, f'output_{filename}.dlg')
        dest_path = os.path.join(".", filename + ".pdbqt")     
        
        shutil.move(pdbqt_file, dest_path)
        
        # Prepare receptor & native, gpf, autogrid4, dpf, autodock4
        run_command(f'{prepare_receptor} -r {receptor_name}.pdb -o {filename}_{receptor_name}.pdbqt -A checkhydrogens') 
        run_command(f'{prepare_gpf} -l {filename}.pdbqt -r {filename}_{receptor_name}.pdbqt -i {lig_name}_{receptor_name}.gpf -o {filename}_{receptor_name}.gpf')
        run_command(f'autogrid4 -p {filename}_{receptor_name}.gpf -l {filename}_{receptor_name}.glg')
        run_command(f'{prepare_dpf} -l {filename}.pdbqt -r {filename}_{receptor_name}.pdbqt {parameter} -i {lig_name}_{receptor_name}.dpf -o {filename}_{receptor_name}.dpf')
        run_command(f'{ensamble} -p {filename}_{receptor_name}.dpf -l {output_file}') 
        read_energy_and_save_to_csv(counter, output_file, csv_path, filename)
      
        # Remove the original and intermediate files
        file_to_remove = glob.glob(os.path.join(directory, f'{filename}*'))
        for file in file_to_remove:
            os.remove(file)

            
    except Exception as e:
        print(f"An error occurred while processing {filename}: {e}")

def process_docking_ligand_test(counter, prepare_receptor, prepare_ligand, prepare_gpf, prepare_dpf, lig_name, output_model_dir, receptor_name, ligand_tmp_dir, csv_path, parameter, ensamble, directory, max_workers):
    os.chdir(directory)
    ligand_files = glob.glob(os.path.join(ligand_tmp_dir, '*.pdbqt'))
    num_ligand_files = len(ligand_files)
    
    counter ["value"] += 1
    if num_ligand_files == 0:        
        return

    futures = []
    
    with concurrent.futures.ProcessPoolExecutor(max_workers=max_workers) as executor:
        for pdbqt_file in ligand_files:
            futures.append(executor.submit(
                process_docking_ligand, directory, prepare_gpf, prepare_dpf, pdbqt_file, counter, prepare_receptor, receptor_name, lig_name, parameter, ensamble, output_model_dir, csv_path))

        for future in tqdm(concurrent.futures.as_completed(futures), total=len(futures), desc="  Docking & save energy"):
            pass  # Tidak perlu melakukan apa-apa, hanya menunggu tugas selesai

def sort_and_rewrite_csv(csv_path):
    try:
        data = []
        with open(csv_path, 'r') as csv_file:
            csv_reader = csv.reader(csv_file)
            header = next(csv_reader)  # Baca header (baris pertama)
            for row in csv_reader:
                data.append(row)            

        # Mengurutkan data berdasarkan kolom kedua (Binding Affinity)
        sorted_data = sorted(data, key=lambda x: float(x[2]))  # Menggunakan lambda untuk mengakses kolom kedua (indeks 2) sebagai kunci pengurutan

        # Menulis data yang sudah diurutkan kembali ke file CSV
        with open(csv_path, 'w', newline='') as csv_file:
            csv_writer = csv.writer(csv_file)
            csv_file.write("No., Ligand ID (name), Binding Affinity (kcal per mol)\n")  # Menulis header kembali
            for index, row in enumerate(sorted_data, start=1):
                csv_writer.writerow([index] + row[1:])  # Menulis nomor urutan, nama, dan binding affinity

        print(f"Output CSV file is in the {csv_path}")                
        return True
    except Exception as e:
        print(f"Error while sorting and rewriting CSV: {e}")
        return False



def print_dev(developer_note, developer_contact, citation_list):
    print("")
    print(developer_note)
    print("")
    print(developer_contact)
    print("")
    print(citation_list)
    print("")

def main(): 
    print_dev(developer_note, developer_contact, citation_list)
    
    os.chdir(current_directory)
    config_file = 'config_ladock4.txt'
    
    with open(config_file, 'r') as config_file:
        config_lines = config_file.readlines()
    
    config_variables = {}
    retry_count = 0
    
    for line in config_lines:
        if not line.strip().startswith('#'):
            try:
                name, value = line.strip().split('=')
                name = name.strip()
                value = value.strip()
                config_variables[name] = value
            except ValueError:
                pass

    ensamble = "autodock4"
    spacing = config_variables['spacing']
    npts = config_variables['npts']
    ga_num_evals = config_variables['ga_num_evals']
    ga_pop_size = config_variables['ga_pop_size']
    ga_run = os.cpu_count()
    num_iterations = 1000 
    max_workers = os.cpu_count()     
    
    mgl_directory = config_variables['mgl_directory']
    prepare_ligand = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "prepare_ligand4.py")
    prepare_receptor = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "prepare_receptor4.py")
    prepare_gpf = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "prepare_gpf4.py")
    prepare_dpf = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "prepare_dpf4.py")
    prepare_lowest_energy = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "write_lowest_energy_ligand.py")
    prepare_summarize_result = os.path.join(mgl_directory, "MGLToolsPckgs", "AutoDockTools", "Utilities24", "summarize_results4.py")
       
    parameter = f' -p ga_num_evals={ga_num_evals} -p ga_pop_size={ga_pop_size} -p ga_run={ga_run}'
    
    ligand_dir = os.path.join(current_directory, 'ligand_input')
    ligand_tmp_dir = os.path.join(current_directory, 'ligand_tmp')
    output_dir = os.path.join(current_directory, 'output')
    
    print("Removing unnecessary files and directories in job directory...")

    # Create ligand_tmp and output directory if it doesn't exist
    if os.path.exists(ligand_tmp_dir):
        shutil.rmtree(ligand_tmp_dir)
    os.makedirs(ligand_tmp_dir, exist_ok=True)    
    if os.path.exists(output_dir):
        shutil.rmtree(output_dir)
    os.makedirs(output_dir, exist_ok=True) 
    
    for directory in glob.glob(os.path.join(current_directory, 'model*')):
        if os.path.isdir(directory):
            counter = {"value": 0}
            dir_name = os.path.basename(directory)
            receptor_name = ""
            lig_name = ""
            print(f"\nProcessing {dir_name.upper()}")
            os.chdir(directory)
            receptor_files = glob.glob('*rec.pdb')
            if receptor_files:    
                receptor_name = os.path.splitext(os.path.basename(receptor_files[0]))[0]
            ligand_files = glob.glob('*lig.pdb')
            if ligand_files:    
                lig_name = os.path.splitext(os.path.basename(ligand_files[0]))[0]                
            output_model_dir = os.path.join(output_dir, dir_name)         
            os.makedirs(output_model_dir, exist_ok=True)
            csv_path = os.path.join(output_model_dir, 'energy_summary.csv')
                
            if os.path.exists(f"{receptor_name}.pdb") and os.path.exists(f"{lig_name}.pdb"):
                with open(csv_path, 'w') as csv_file:
                    csv_file.write("No., Ligand ID (name), Binding Affinity (kcal per mol)\n")
                os.chdir(directory)
                print("Docking reference ligand:")
                print(f"  Receptor: {receptor_name}.pdbqt")
                print(f"  Ligand: {lig_name}.pdbqt")
                process_docking_ligand_native(prepare_gpf, prepare_dpf, counter, spacing, npts, max_workers, num_iterations, csv_path, prepare_receptor, receptor_name, prepare_ligand, lig_name, directory, ensamble, output_model_dir, parameter)
                print("\nDocking test ligands: ")
                print(f"  Receptor: {receptor_name}.pdbqt")                  
                os.chdir(ligand_dir)
                
                valid_extensions = (".txt")
                ligand_link_files = [f for f in os.listdir(ligand_dir) if any(f.endswith(ext) for ext in valid_extensions)]
                for ligand_link_path in ligand_link_files:
                    if os.path.exists(ligand_link_path):
                        with open(ligand_link_path, 'r') as f:
                            lines = f.readlines()                        
                        for line in lines:
                            if line.strip():
                                if not line.startswith("#"):
                                    ligand_url = line.strip()
                                    process_ligand_link(prepare_gpf, prepare_dpf, ligand_url, ligand_dir, ligand_tmp_dir, lig_name, num_iterations, max_workers, spacing, npts, csv_path, prepare_receptor, receptor_name, prepare_ligand, directory, ensamble, output_model_dir, counter, parameter)
                    
                valid_extensions = (".sdf", ".smi", ".pdb", ".smiles", ".mol", ".mol2")
                ligand_files = [f for f in os.listdir(ligand_dir) if any(f.endswith(ext) for ext in valid_extensions)]
                for ligand_file in ligand_files: 
                    process_in_ligand_dir(ligand_file, ligand_dir, ligand_tmp_dir)
                    
                    valid_extensions = (".sdf", ".smi")
                    ligand_tmp_files = [f for f in os.listdir(ligand_tmp_dir) if any(f.endswith(ext) for ext in valid_extensions)]
                    num_ligand_files = len(ligand_tmp_files)
                    print(f"  Total molecules = {num_ligand_files}")                    
                    print(f"  Using {lig_name}.pdb docking parameter as reference")
                    for ligand_tmp_file in ligand_tmp_files:
                        process_in_ligand_tmp_dir(prepare_ligand, ligand_tmp_file, ligand_tmp_dir, max_workers)                        
                        process_docking_ligand_test(counter, prepare_receptor, prepare_ligand, prepare_gpf, prepare_dpf,  lig_name, output_model_dir, receptor_name, ligand_tmp_dir, csv_path, parameter, ensamble, directory, max_workers)
                            
                sort_and_rewrite_csv(csv_path)               
                print(f"Success: {dir_name.upper()}")

            else:
                print(f"Skipping docking: receptor or reference ligand in {dir_name} not found.")
                os.chdir(current_directory)

    # Print developer's note, contact, and citation listdir
    print_dev(developer_note, developer_contact, citation_list)

if __name__ == "__main__":
    current_directory =  (os.path.join(os.getcwd(), "LADOCK_ladock4")) 
    if os.path.exists(current_directory):  
        main() 
    else:
        print("Your job directory (LADOCK_ladock4) is not ready. Please create it using:")
        print("ladock --create ladock4")



#!/usr/bin/env python3

import os
import sys
import argparse
import glob
import shutil
import subprocess

current_directory = os.getcwd()
source_directory = os.path.dirname(os.path.abspath(__file__))
config_directory = os.path.join(source_directory, "config")
share_directory = os.path.join(source_directory, "share")
mdp_directory = os.path.join(source_directory, "share", "mdp")
vina = os.path.join(share_directory, "vina_1.2.5_linux_x86_64")
vina_split = os.path.join(share_directory, "vina_split_1.2.5_linux_x86_64")

dn = os.path.join(source_directory, "developerNote.txt")
with open(dn, 'r') as file:
    content = file.read()

# Pisahkan isi file menjadi blok-blok yang sesuai dengan variabel-variabel
blocks = content.split('\n\n')
# Inisialisasi variabel
developer_note = ""
developer_contact = ""
citation_list = ""

# Loop melalui blok-blok dan mengisi variabel yang sesuai
for block in blocks:
    if block.startswith("developer_note ="):
        developer_note = block[block.find("=") + 1:].strip()
    elif block.startswith("developer_contact ="):
        developer_contact = block[block.find("=") + 1:].strip()
    elif block.startswith("citation_list ="):
        citation_list = block[block.find("=") + 1:].strip()

def create_lavina():
    # Membuat struktur direktori dan file konfigurasi di dalamnya
    if not os.path.exists('LADOCK_lavina'):
        os.mkdir('LADOCK_lavina')
    os.chdir('LADOCK_lavina')

    if not os.path.exists('ligand_input'):
        os.mkdir('ligand_input')
   
    for i in range(1, 4):  # Membuat direktori model_01, model_02, model_03, model_04
        model_dir = f'model_{i:02d}'
        if not os.path.exists(model_dir):
            os.mkdir(model_dir)

    # Mengambil isi 'config_content' dari file 'lavinaConfig.py'
    from config.lavinaConfig import config_content
    with open('config_lavina.txt', 'w') as config_file:
        config_file.write(config_content)

    # Mengambil isi 'ligand_link_default' dari file 'lavinaConfig.py'
    from config.lavinaConfig import ligand_link_default
    ligand_input_dir = os.path.join('ligand_input', 'ligand_link.txt')   
    with open(ligand_input_dir, 'w') as ligand_link_file:
        ligand_link_file.write(ligand_link_default) 
 
def create_lavinagpu():
    # Membuat struktur direktori dan file konfigurasi di dalamnya
    if not os.path.exists('LADOCK_lavinagpu'):
        os.mkdir('LADOCK_lavinagpu')
    os.chdir('LADOCK_lavinagpu')

    if not os.path.exists('ligand_input'):
        os.mkdir('ligand_input')
   
    for i in range(1, 4):  # Membuat direktori model_01, model_02, model_03, model_04
        model_dir = f'model_{i:02d}'
        if not os.path.exists(model_dir):
            os.mkdir(model_dir)

    # Mengambil isi 'config_content' dari file 'lavinagpuConfig.py'
    from config.lavinagpuConfig import config_content
    with open('config_lavinagpu.txt', 'w') as config_file:
        config_file.write(config_content)

    # Mengambil isi 'ligand_link_default' dari file 'lavinaConfig.py'
    from config.lavinagpuConfig import ligand_link_default
    ligand_input_dir = os.path.join('ligand_input', 'ligand_link.txt')   
    with open(ligand_input_dir, 'w') as ligand_link_file:
        ligand_link_file.write(ligand_link_default) 
        
def create_la2vina():
    # Membuat struktur direktori dan file konfigurasi di dalamnya
    if not os.path.exists('LADOCK_la2vina'):
        os.mkdir('LADOCK_la2vina')
    os.chdir('LADOCK_la2vina')

    if not os.path.exists('ligand_input'):
        os.mkdir('ligand_input')

    if not os.path.exists('main_ligand'):
        os.mkdir('main_ligand')
    
    for i in range(1, 4):  # Membuat direktori model_01, model_02, model_03, model_04
        model_dir = f'model_{i:02d}'
        if not os.path.exists(model_dir):
            os.mkdir(model_dir)

    # Mengambil isi 'config_content' dari file 'la2vinaConfig.py'
    from config.la2vinaConfig import config_content
    with open('config_la2vina.txt', 'w') as config_file:
        config_file.write(config_content)

    # Mengambil isi 'ligand_link_default' dari file 'lavinaConfig.py'
    from config.lavinaConfig import ligand_link_default
    
    ligand_input_dir = os.path.join('ligand_input', 'ligand_link.txt')   
    with open(ligand_input_dir, 'w') as ligand_link_file:
        ligand_link_file.write(ligand_link_default)
       
def create_ladock4():
    # Membuat struktur direktori dan file konfigurasi di dalamnya
    if not os.path.exists('LADOCK_ladock4'):
        os.mkdir('LADOCK_ladock4')
    os.chdir('LADOCK_ladock4')

    if not os.path.exists('ligand_input'):
        os.mkdir('ligand_input')
    
    for i in range(1, 4):  # Membuat direktori model_01, model_02, model_03, model_04
        model_dir = f'model_{i:02d}'
        if not os.path.exists(model_dir):
            os.mkdir(model_dir)

    # Mengambil isi 'config_content' dari file 'ladock4Config.py'
    from config.ladock4Config import config_content
    with open('config_ladock4.txt', 'w') as config_file:
        config_file.write(config_content)

    # Mengambil isi 'ligand_link_default' dari file 'ladock4Config.py'
    from config.ladock4Config import ligand_link_default
    ligand_input_dir = os.path.join('ligand_input', 'ligand_link.txt')   
    
    with open(ligand_input_dir, 'w') as ligand_link_file:
        ligand_link_file.write(ligand_link_default)
        
def create_ladockgpu():
    # Membuat struktur direktori dan file konfigurasi di dalamnya
    if not os.path.exists('LADOCK_ladockGPU'):
        os.mkdir('LADOCK_ladockGPU')
    os.chdir('LADOCK_ladockGPU')

    if not os.path.exists('ligand_input'):
        os.mkdir('ligand_input')
    
    for i in range(1, 4):  # Membuat direktori model_01, model_02, model_03, model_04
        model_dir = f'model_{i:02d}'
        if not os.path.exists(model_dir):
            os.mkdir(model_dir)

    # Mengambil isi 'config_content' dari file 'ladock4Config.py'
    from config.ladockgpuConfig import config_content
    with open('config_ladockgpu.txt', 'w') as config_file:
        config_file.write(config_content)

    # Mengambil isi 'ligand_link_default' dari file 'ladockGPUConfig.py'
    from config.ladockgpuConfig import ligand_link_default
    ligand_input_dir = os.path.join('ligand_input', 'ligand_link.txt')   
    
    with open(ligand_input_dir, 'w') as ligand_link_file:
        ligand_link_file.write(ligand_link_default)      

def create_gmxprolig():
    # Membuat struktur direktori dan file konfigurasi di dalamnya
    if not os.path.exists('LADOCK_gmxprolig'):
        os.mkdir('LADOCK_gmxprolig')
    os.chdir('LADOCK_gmxprolig')
   
    for i in range(1, 4):  # Membuat direktori model_01, model_02, model_03, model_04
        model_dir = f'complex_{i:02d}'
        if not os.path.exists(model_dir):
            os.mkdir(model_dir)

    # Menyalin file *.mdp dari direktori source ke direktori tujuan
    source_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "share", "mdp")
    destination_dir = os.getcwd()

    if not os.path.exists(destination_dir):
        os.mkdir(destination_dir)

    for filename in os.listdir(source_dir):
        if filename.endswith(".mdp"):
            source_file = os.path.join(source_dir, filename)
            destination_file = os.path.join(destination_dir, filename)
            shutil.copy(source_file, destination_file)

    # Mengambil isi 'config_content' dari file 'lagmxConfig.py'
    from config.gmxproligConfig import config_content
    with open('config_gmxprolig.txt', 'w') as config_file:
        config_file.write(config_content)
        
def create_gmxliglig():
    # Membuat struktur direktori dan file konfigurasi di dalamnya
    if not os.path.exists('LADOCK_gmxliglig'):
        os.mkdir('LADOCK_gmxliglig')
    os.chdir('LADOCK_gmxliglig')
    
    for i in range(1, 4):  # Membuat direktori model_01, model_02, model_03, model_04
        model_dir = f'complex_{i:02d}'
        if not os.path.exists(model_dir):
            os.mkdir(model_dir)
            
    # Menyalin file *.mdp dari direktori source ke direktori tujuan
    source_dir = os.path.join(os.path.dirname(os.path.abspath(__file__)), "share", "mdp")
    destination_dir = os.getcwd()

    if not os.path.exists(destination_dir):
        os.mkdir(destination_dir)

    for filename in os.listdir(source_dir):
        if filename.endswith(".mdp"):
            source_file = os.path.join(source_dir, filename)
            destination_file = os.path.join(destination_dir, filename)
            shutil.copy(source_file, destination_file)

    # Mengambil isi 'config_content' dari file 'ligligConfig.py'
    from config.ladyna3ligConfig import config_content
    with open('config_gmxliglig.txt', 'w') as config_file:
        config_file.write(config_content)

def main():
    parser = argparse.ArgumentParser(description='LADOCK is an innovative and free tool designed for conducting simultaneous simulations in computer-aided drug discovery, encompassing molecular docking and molecular dynamics. In molecular docking, LADOCK excels in handling single or double ligands. It supports ligands from various online sources. In molecular dynamics, LADOCK efficiently manages protein-ligand interactions and even ligand-ligand interactions, accommodating scenarios with one or multiple proteins and ligands.')
    parser.add_argument('--version', action='version', version='ladock 1.0')
    parser.add_argument('--create', choices=['lavina', 'lavinagpu', 'la2vina', 'ladock4', 'ladockgpu', 'gmxprolig', 'gmxliglig'], help='Create necessary input')
    parser.add_argument('--run', choices=['lavina', 'lavinagpu', 'la2vina', 'ladock4', 'ladockgpu', 'gmxprolig', 'gmxliglig'], help='Execute the specific simulation: lavina (AutoDock Vina), la2vina (MLSD with Autodock Vina), ladock4 (AutoDock4), ladockgpu (AutoDock-GPU), gmxprolig (gromacs for 1 protein with 1 or multiple ligands), gmxliglig (gromacs multiple ligands)' )
    args = parser.parse_args()

    if args.create:
        create_input(args.create)

    elif args.run:
        run_simulation(args.run)
    
    else:
        print("Invalid option. Please use one of the following:")
        print("'--create lavina' to create input for AutoDock Vina simulation.")
        print("'--create lavinagpu' to create input for AutoDock Vina GPU simulation.")
        print("'--create la2vina' to create input for AutoDock Vina simulation using multiple ligand simultaneously molecular-docking technique).")
        print("'--create ladock4' to create input for AutoDock4 simulation.")
        print("'--create ladockgpu' to create input for AutoDock4-GPU simulation.")
        print("'--create gmxprolig' to create input for single or multiple ligand simultaneously molecular dynamics simulation with GROMACS.")
        print("'--create gmxliglig' to create input for single/multiple ligand and single/multiple (chain) protein simultaneously molecular dynamics simulation with GROMACS")
        print("'--run lavina' for docking simulation with AutoDock Vina.")
        print("'--run la2vina' for docking simulation with AutoDock Vina using MLSD (multiple ligand simulation docking technique).")
        print("'--run ladock4' for docking simulation with AutoDock4.")
        print("'--run ladockgpu' for docking simulation with AutoDock-GPU.")
        print("'--run gmxprolig' for single or multiple ligand simultaneously molecular dynamics simulation with GROMACS.")
        print("'--run gmxliglig' for single/multiple ligand and single/multiple (chain) protein simultaneously molecular dynamics simulation with GROMACS")
        print("'--version' to show the LADOCK's version number.")
        print("'--help' or '-h' for help.")
        print("\nNote: Ensure that the following dependency packages are installed on your system:")
        print("* autodock4")
        print("* autodock-gpu")
        print("* autodock vina")
        print("* vina-gpu")
        print("* mgltools")
        print("* gromacs")
        print("* acpype")

def create_input(simulation_type):
    if simulation_type == 'lavina':
        create_lavina()
        print("LADOCK_lavina, subdirectories, 'config_lavina.txt', and 'ligand_link.txt' have been successfully created in the 'LADOCK_lavina' directory. Please edit them according to your needs.")
    
    if simulation_type == 'lavinagpu':   
        create_lavinagpu()
        print("LADOCK_lavinagpu, subdirectories, 'config_lavina.txt', and 'ligand_link.txt' have been successfully created in the 'LADOCK_lavinagpu' directory. Please edit them according to your needs.")

    elif simulation_type == 'la2vina':
        create_la2vina()
        print("LADOCK_la2vina, subdirectories, and 'config_la2vina.txt' have been successfully created in the 'LADOCK_la2vina' directory. Please edit them according to your needs.")

    elif simulation_type == 'ladock4':
        create_ladock4()
        print("LADOCK_ladock4, subdirectories, 'config_ladock4.txt', and 'ligand_link.txt' have been successfully created in the 'LADOCK_ladock4' directory. Please edit them according to your needs.")

    elif simulation_type == 'ladockgpu':
        create_ladockgpu()
        print("LADOCK_ladockgpu, subdirectories, 'config_ladockgpu.txt', and 'ligand_link.txt' have been successfully created in the 'LADOCK_ladock4' directory. Please edit them according to your needs.")

    elif simulation_type == 'gmxprolig':
        create_lagmx()
        print("LADOCK_gmxprolig and subdirectory created successfully.")
       
    elif simulation_type == 'gmxliglig':
        create_ladyna3lig()
        print("LADOCK_gmxliglig and subdirectory created successfully.")

def run_simulation(simulation_type):
    
    if simulation_type == 'lavina':
        import config.lavinaConfig
        current_dir = os.path.dirname(os.path.abspath(__file__))
        path = os.path.join(current_dir, 'lavina.py')
        subprocess.run(['python3', path])
        
    if simulation_type == 'lavinagpu':
        import config.lavinagpuConfig
        current_dir = os.path.dirname(os.path.abspath(__file__))
        path = os.path.join(current_dir, 'lavinagpu.py')
        subprocess.run(['python3', path])

    elif simulation_type == 'la2vina':
        import config.la2vinaConfig
        current_dir = os.path.dirname(os.path.abspath(__file__))
        path = os.path.join(current_dir, 'la2vina.py')
        subprocess.run(['python3', path])

    elif simulation_type == 'ladock4':
        import config.ladock4Config
        current_dir = os.path.dirname(os.path.abspath(__file__))
        path = os.path.join(current_dir, 'ladock4.py')
        subprocess.run(['python3', path])

    elif simulation_type == 'ladockgpu':
        import config.ladockgpuConfig
        current_dir = os.path.dirname(os.path.abspath(__file__))
        path = os.path.join(current_dir, 'ladockgpu.py')
        subprocess.run(['python3', path])

    elif simulation_type == 'gmxprolig':
        import config.gmxproligConfig
        current_dir = os.path.dirname(os.path.abspath(__file__))   
        path = os.path.join(current_dir, 'gmxprolig.py')
        subprocess.run(['python3', path])        
       
    elif simulation_type == 'gmxliglig':
        import config.gmxligligConfig
        current_dir = os.path.dirname(os.path.abspath(__file__))
        path = os.path.join(current_dir, 'gmxliglig.py')
        subprocess.run(['python3', path])
        
    elif simulation_type == 'test':
        import config.gmxligligConfig
        current_dir = os.path.dirname(os.path.abspath(__file__))
        path = os.path.join(current_dir, 'test.py')
        subprocess.run(['python3', path])
        
if __name__ == "__main__":
    main()
